\name{allsubhypercubes}
\alias{allsubhypercubes}
\title{Subhypercubes of magic hypercubes}
\description{
Extracts all subhypercubes from an n-dimensional hypercube.
}
\usage{
allsubhypercubes(a)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{a}{The magic hypercube whose subhypercubes are computed}
}
\value{
  Returns a list, each element of which is a subhypercube.  Note that
  major diagonals are also returned (as n-by-1 arrays).
}
\author{Robin K. S. Hankin}
\note{This function is a dog's dinner.  It's complicated, convoluted,
  and needs two absurd uses of the \code{eval(parse(text=...))}
  construction.  Basically it sucks big time.

  BUT\ldots I cannot for the life of me see a better way that gives the
  same results, without loops, on hypercubes of arbitrary dimension.

  The function cannot cope with \code{d} as high as 5, for \code{n=4}.
  Heigh ho.
  }
\seealso{\code{\link{is.perfect}}}
\examples{
  library(abind)
  a <- magichypercube.4n(1,d=4)
  allsubhypercubes(a)
}
\keyword{array}

\eof
\name{allsums}
\alias{allsums}
\title{Row, column, and two diagonal sums of arrays}
\description{
  Returns all rowsums, all columnsums, and all (broken) diagonal sums of
  a putative magic square.
}
\usage{
allsums(m)
}
\arguments{
  \item{m}{The square to be tested}
}
\value{
  Returns a list of four elements.
  \item{rowsums}{All n row sums}
  \item{colsums}{All n column sums}
  \item{majors}{All n broken major diagonals (northwest-southeast).
    First element is the long (unbroken) major diagonal, tested by
    \code{is.magic()}}
  \item{minors}{All n broken minor diagonals (northeast-southwest).
    First element is the long (unbroken) minor diagonal.}
}
\author{Robin K. S. Hankin}
\note{Used by \code{is.magic()} et seq}
\seealso{\code{\link{is.magic}},\
\code{\link{is.semimagic}},\
\code{\link{is.panmagic}}
  }
\examples{
allsums(magic.prime(7))
}
\keyword{array}

\eof
\name{as.standard}
\alias{as.standard}
\alias{is.standard}

\title{Standard form for magic squares}
\description{
  Transforms a magic square into Frenicle's standard form.
}
\usage{
as.standard(m)
is.standard(m)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{m}{Magic square to be tested or transformed}
}

\details{
\code{as.standard()} transforms a magic square into Frenicle's standard
form.  The four numbers at each of the four corners are determined.
First, the square is rotated so the smallest of the four is at the upper
left.  Then, element \code{[1,2]} is compared with element\code{[2,1]}
and, if it is larger, the transpose is taken.

Thus all eight rotated and transposed versions of a magic square have
the same standard form.

The square returned by \code{magic()} is in standard form.

\code{is.standard()} returns \code{TRUE} if the magic square is in
standard form.  This function checks for neither magicness nor normality
(use \code{\link{is.magic}} and \code{\link{is.normal}} for this).
}

\author{Robin K. S. Hankin}
\seealso{\code{\link{magic}},\ \code{\link{eq}}}
\examples{
is.standard(magic.2np1(4))
as.standard(magic.4n(3))
}
\keyword{array}

\eof
\name{diag.off}
\alias{diag.off}
\title{Extracts broken diagonals}
\description{
  Returns broken diagonals of a magic square
}
\usage{
diag.off(a, offset = 0, nw.se = TRUE)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{a}{Square matrix}
  \item{offset}{vertical offset} 
  \item{nw.se}{Boolean variable with TRUE meaning trace diagonals along
    the northwest-southeast direction (point \code{[1,n]}
    to \code{[n,1]}).}  }
\details{
  Useful when testing for panmagic squares.  The first element is always
  the unbroken one (ie \code{[1,1]} to \code{[n,n]} if \code{nw.se} is
  \code{TRUE} and \code{[1,n]} to \code{[n,1]} if \code{nw.se} is
  \code{FALSE}.
}
\author{Robin K. S. Hankin}
\seealso{\code{\link{is.panmagic}}}
\examples{
diag.off(magic(10),nw.se=FALSE,offset=0)
diag.off(magic(10),nw.se=FALSE,offset=1)
}
\keyword{array}

\eof
\name{dimension}
\alias{dimension}
\title{Generalized row and col}
\description{
Given an array, returns an array of the same size whose elements
are sequentially numbered along the \eqn{i^{\rm th}}{i-th} dimension.
}
\usage{
dimension(a, i)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{a}{array to be converted}
  \item{i}{Number of the dimension}
}
\value{
An integer matrix with the same dimensions as \code{a}, with element 
\eqn{\left(n_1,n_2,\ldots n_d\right)^{\rm th}}{(n1,n2, ..., n_d)}
being \eqn{n_i}{n_i}.
}
\author{Robin K. S. Hankin}
\examples{
a <- array(0,c(3,3,2,2))
dimension(a,2)
a[(dimension(a,1)+dimension(a,2)+dimension(a,3)+dimension(a,4))\%\%2==1] <- 1
a
}
\keyword{array}

\eof
 \name{eq}
\alias{eq}
\alias{ne}
\alias{gt}
\alias{lt}
\alias{ge}
\alias{le}
\alias{%eq%}
\alias{%ne%}
\alias{%gt%}
\alias{%lt%}
\alias{%ge%}
\alias{%le%}
\title{Comparison of two magic squares}
\description{
Compares two magic squares according to Frenicle's method.  Mnemonic is
the old Fortran ``.GT."  (for ``Greater Than") comparison et seq.

To compare magic square \code{a} with magic square \code{b}, their
elements are compared in rowwise order: \code{a[1,1]} is compared with
\code{b[1,1]} , then \code{a[1,2]} with \code{b[1,2]}, up to
\code{a[n,n]}.  Consider the first element that is different, say
\code{[i,j]}.  Then \code{a<b} if \code{a[i,j]<b[i,j]}.
}
\usage{
eq(m1, m2)
ne(m1, m2)
gt(m1, m2)
lt(m1, m2)
ge(m1, m2)
le(m1, m2)
}
\arguments{
  \item{m1}{First magic square}
  \item{m2}{Second magic square}
}
\author{Robin K. S. Hankin}
\note{Rather clumsy function definition due to the degenerate case of 
testing two identical matrices (\code{min(NULL)} is undefined).
}
\seealso{\code{\link{as.standard}}}
\examples{
#magic(4) %eq% magic.4n(1)
eq(magic(4) , magic.4n(1))
}
\keyword{array}

\eof
\name{force.integer}
\alias{force.integer}
\title{Integerize array elements}
\description{
Returns an elementwise \code{as.integer}-ed array.  All magic squares
should have integer elements.
}
\usage{
force.integer(x)
}
\arguments{
  \item{x}{Array to be converted}
}
\author{Robin K. S. Hankin}
\keyword{array}

\eof
\name{hendricks}
\alias{hendricks}
\docType{data}
\title{A perfect magic cube due to Hendricks}
\description{
A perfect \eqn{8\times 8\times 8}{8*8*8} magic cube due to Hendricks}
\usage{data(hendricks)}
\source{\code{http://members.shaw.ca/hdhcubes/cube_perfect.htm#Definitions}}
\examples{
data(hendricks)
is.perfect(hendricks)
}
\keyword{datasets}
\keyword{array}

\eof
\name{is.magic}
\alias{is.magic}
\alias{is.panmagic}
\alias{is.semimagic}
\alias{is.associative}
\alias{is.normal}
\title{Various tests for the magicity of a square}
\description{
  Returns TRUE if the square is magic, semimagic, panmagic, associative,
  normal.
}
\usage{
is.magic(m,give.answers=FALSE)
is.panmagic(m,give.answers=FALSE)
is.semimagic(m,give.answers=FALSE)
is.associative(m)
is.normal(m)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{m}{The square to be tested}
  \item{give.answers}{Boolean, with TRUE meaning return a list of
    rowwise, columnwise and both broken diagonal (NW-SE and NE-SW) sums.}
}
\details{
  A semimagic square is one whose row sums all equal its columnwise
  sums (ie the magic constant).

  A magic square is a semimagic square with the sum of both unbroken
  diagonals equal to the magic constant.

  A panmagic square is a magic square all of whose broken diagonals have
  sum to the magic constant.

  A normal square is one that contains \eqn{n^2}{n^2} consecutive
  integers (typically starting at 0 or 1).  
}
\value{
  Returns TRUE if the square is semimagic, etc.

  If \code{give.answers} is TRUE, return a list of five elements.  The
  first is TRUE if the square is (semimagic, magic, panmagic), elements
  2-5 give the result of a call to \code{allsums()}, viz: rowwise and
  columwise sums; and broken major
  (ie NW-SE) and minor (ie NE-SW) diagonal sums.}
\references{\code{http://mathworld.wolfram.com/MagicSquare.html}}
\author{Robin K. S. Hankin}
\seealso{\code{\link{min.max}},\code{\link{is.perfect}},\code{\link{is.semimagichypercube}}}
\examples{
is.magic(magic(4))
f <- function(n){is.magic(magic(n))}
all(sapply(3:50,f))

is.panmagic(panmagic.4())
is.panmagic(panmagic.8())

proper.magic <- function(m){is.magic(m) & min.max(c(1,diff(sort(m))))}
proper.magic(magic(20))
}
\keyword{array}

\eof
\name{is.semimagichypercube}
\alias{is.semimagichypercube}
\alias{is.magichypercube}
\alias{is.perfect}
\title{magic hypercubes}
\description{
Returns \code{TRUE} if a hypercube is semimagic.
}
\usage{
is.semimagichypercube(a)
is.magichypercube(a)
is.perfect(a)
}

\arguments{
  \item{a}{The hypercube (array) to be tested.}
}
\details{

  (Although apparently non-standard, here a hypercube is defined to have
  dimension \eqn{d}{d} and order \eqn{n}{n}---and thus has \eqn{n^d}{n^d}
  elements).
  
  A semimagic hypercube has all ``rook's move'' sums equal to the magic
  constant.

  A magic hypercube is a semimagic hypercube with all
  \eqn{2^{d-1}}{2^(d-1)} major (ie extreme point-to-extreme point)
  diagonals summing correctly.  Note that not all
  hyperplane subsets are necessarily magic! (for example, consider a
  5-dimensional magic hypercube \code{a}.  The square \code{b} defined
  by \code{a[1,1,1,,]} might not be semimagic: the rows and columns of
  \code{b} are not covered by the definition of a magic hypercube.

  A perfect magic hypercube is a magic hypercube with all nonbroken
  diagonals summing correctly.  This is a bad-ass requirement for high
  dimensional hypercubes.

  Even in three dimensions, being perfect is pretty bad.  Consider a
  \eqn{5\times5\times 5}{5x5x5} (ie three dimensional), cube.  Say
  \code{a=magiccube.2np1(2)}.  Then the square defined by
  \code{sapply(1:n,function(i){a[,i,6-i]}, simplify=TRUE)}, which is a
  subhypercube of \code{a}, is not even semimagic: the
  rowsums are incorrect (the colsums must sum correctly because \code{a}
  is magic).  Note that the diagonals of this square are two of the
  ``extreme point-to-point'' diagonals of \code{a}.

  A pandiagonal magic hypercube (or sometimes just a perfect hypercube)
  has sums equal, and all sums of all diagonals, including broken
  diagonals, are equal.  This is one seriously bad-ass requirement.  I
  reckon that is a total of
  \eqn{\frac{1}{2}\left( 3^d-1\right)\cdot n^{d-1}}{(3^d-1)n^(d-1)/2}
  correct summations.  I can't see how to code this up in anything like
  a vectorized manner.

  The terminology in this area is pretty confusing.  
  }
\references{\code{http://members.shaw.ca/hdhcubes/cube_perfect.htm#Cubes}}
\author{Robin K. S. Hankin}
\seealso{\code{\link{is.magic}}, \code{\link{allsubhypercubes}},\code{\link{hendricks}}}
\examples{
library(abind)
is.semimagichypercube(magiccube.2np1(1))
is.semimagichypercube(magichypercube.4n(1,d=4))

is.perfect(magichypercube.4n(1,d=4))

data(hendricks)
is.perfect(hendricks)


#note that Hendricks' magic cube also has many broken diagonals summing
#correctly:

a <- allsubhypercubes(hendricks)
ld <- function(a){length(dim(a))}

jj <- unlist(lapply(a,ld))
f <- function(i){is.perfect(a[[which(jj==2)[i]]])}
all(sapply(1:sum(jj==2),f))

#but this is NOT enough to ensure that it is pandiagonal (but I
#think hendricks is pandiagonal).

}
\keyword{array}

\eof
\name{magic.2np1}
\alias{magic.2np1}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Magic squares of odd order}
\description{
  Function to create magic squares of odd order 
}
\usage{
magic.2np1(m, ord.vec = c(-1, 1), break.vec = c(1, 0), start.point=NULL)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{m}{creates a magic square of order \eqn{n=2m+1}{n=2m+1}}
  \item{ord.vec}{ordinary vector.  Default value of \code{c(-1,1) }
    corresponds to the usual northeast direction}
  \item{break.vec}{break vector.  Default of \code{c(1,0)} corresponds to the
    usual south direction}
  \item{start.point}{Starting position for the method (ie coordinates of
    unity).  Default value of NULL corresponds to row 1, column \code{m}.}
}
\references{Written up in loads of places.  The method (at least with
  the default ordinary and break vectors) seems to have been known since
  at least the Renaissance.

  Benson and Jacoby, and the Mathematica website, discuss the problem
  with nondefault ordinary and break vectors.
}
\author{Robin K. S. Hankin}
\seealso{\code{\link{magic}}, \code{\link{magic.prime}}}
\examples{
magic.2np1(1)
f <- function(n){is.magic(magic.2np1(n))}
all(sapply(1:20,f))

is.panmagic(magic.2np1(5,ord.vec=c(2,1),break.vec=c(1,3)))
}
\keyword{array}

\eof
\name{magic.4n}
\alias{magic.4n}
\title{Magic squares of order 4n}% \eqn{4n}{4n}}
\description{
Produces an associative magic square of order \eqn{4n}{4n} using the delta-x method.
}
\usage{
magic.4n(m, method="a")
}
\arguments{
  \item{m}{Order \eqn{n}{n} of the square is given by \eqn{n=4m}{n=4m}}
  \item{method}{string specifying which method to use.  Method ``a''
    appears to be uniformly better, and clearer.}
}
\author{Robin K. S. Hankin}
\seealso{\code{\link{magic}}}
\examples{
magic.4n(4)
is.magic(magic.4n(5))
}
\keyword{array}


\eof
\name{magic.4np2}
\alias{magic.4np2}
\title{Magic squares of order 4n+2}
\description{
  Produces a magic square of order \eqn{4n+2}{4n+2} using
  Conway's ``LUX'' method
}
\usage{
magic.4np2(m)
}
\arguments{
  \item{m}{returns a magic square of order \eqn{n=4m+2}{n=4m+2}
    for \eqn{n>1}{n>1}, using Conway's
    ``LUX'' construction.}
}
\references{\code{http://mathworld.wolfram.com/MagicSquare.html}}
\author{Robin K. S. Hankin}
\note{I am not entirely happy with the method used: it's too complicated}
\seealso{\code{\link{magic}}}
\examples{
magic.4np2(1)
is.magic(magic.4np2(3))
  }
}
\keyword{array}

\eof
\name{magic.8}
\alias{magic.8}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Regular magic squares of order 8}
\description{
Returns all 90 regular magic squares of order 8
}
\usage{
magic.8(...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{\dots}{ignored  }
}
\value{
Returns an array of dimensions \code{c(8,8,90)} of which each slice is
an 8-by-8 magic square.
}
\references{\code{http://www.grogono.com/magic/index.php}}
\author{Robin K. S. Hankin}
\keyword{array}

\eof
\name{magic}
\alias{magic}
\title{Creates magic squares}
\description{
  Creates magic squares of any order \eqn{>2}{>2}.  Uses the appropriate
method depending on n modulo 4.
}
\usage{
magic(n)
}
\arguments{
  \item{n}{Order of magic square}
}
\details{
Calls either \code{magic.2np1()}, \code{magic.4n()},
or \code{magic.4np2()} depending on the value of \code{n}.  Returns a
magic square in standard format.
}
\references{William H. Benson and Oswald Jacoby.  New recreations with
  magic squares, Dover 1976.
}
\author{Robin K. S. Hankin}
\seealso{\code{\link{magic.2np1}}, \code{\link{magic.prime}},
  \code{\link{magic.4np2}}, \code{\link{magic.4n}},
  \code{\link{as.standard}}, \code{\link{force.integer}}}
\examples{
magic(6)
all(sapply(3:10,function(n){is.magic(magic(n))}))
}
\keyword{array}

\eof
\name{magic.composite}
\alias{magic.composite}
\title{Composite magic squares}
\description{
  Gives a magic square that is a product of two magic squares.
}
\usage{
magic.composite(a, b)
}
\arguments{
  \item{a}{First magic square; if \code{a} is an integer, use \code{magic(a)}.}
  \item{b}{as above}
}
\details{
Produces a magic square of order order(a)*order(b).
}
\references{
  William H. Benson and Oswald Jacoby. New recreations with magic squares, Dover 1976.}
\author{Robin K. S. Hankin}
\examples{
magic.composite(3,4)
magic.composite(4,3)
}
\keyword{array}

\eof
\name{magic.prime}
\alias{magic.prime}
\title{Magic squares prime order}
\description{
Produces magic squares of order using the standard method}
}
\usage{
magic.prime(n,i=2,j=3)
}
\arguments{
  \item{n}{The order of the square}
  \item{i}{row number of increment}
  \item{j}{column number of increment}
}
\details{
Claimed to work for prime order, but I've tried it (with the defaults
for \code{i} and \code{j}) for many  integers of the
form \eqn{6n+1}{6n+1} and \eqn{6n-1}{6n-1} and found no exceptions;
indeed, they all seem to be panmagic.  It is not clear to me 
when the process works and when it doesn't.  
}
\references{\code{http://www.magic-squares.de/general/general.html}}
\author{Robin K. S. Hankin}
\examples{
magic.prime(7)
f <- function(n){is.magic(magic.prime(n))}
all(sapply(6*1:30+1,f))
all(sapply(6*1:30-1,f))

is.magic(magic.prime(9,i=2,j=4),give.answers=TRUE)
magic.prime(7,i=2,j=4)
}
\keyword{array}

\eof
\name{magiccube.2np1}
\alias{magiccube.2np1}
\title{Magic cubes of order 2n+1}
\description{
Creates odd-order magic cubes
}
\usage{
magiccube.2np1(m)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{m}{n=2m+1}
}
\references{website}
\author{Robin K. S. Hankin}
\seealso{\code{\link{magic}}}

\examples{

#try with m=3, n=2*3+1=7:

m <-7                     
n <- 2*m+1


apply(magiccube.2np1(m),c(1,2),sum)
apply(magiccube.2np1(m),c(1,3),sum)
apply(magiccube.2np1(m),c(2,3),sum)

#major diagonal checks out:
sum(magiccube.2np1(m)[matrix(1:n,n,3)])

#now other diagonals:
b <- c(-1,1)
f <- function(dir,v){if(dir>0){return(v)}else{return(rev(v))}}
g <- function(jj){sum(magiccube.2np1(m)[sapply(jj,f,v=1:n)])}
apply(expand.grid(b,b,b),1,g)  #each diagonal twice, once per direction.
}

\keyword{array}


\eof
\name{magichypercube.4n}
\alias{magichypercube.4n}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Magic hypercubes of order 4n}
\description{
Returns n-dimensional magic cubes of order 4n
}
\usage{
magichypercube.4n(m, d = 3)
}
%- maybe also 'usage' for other objects documented here. 
\arguments{
  \item{m}{Magic hypercube produced of order \eqn{n=4m}{n=4m}}
  \item{d}{Dimensionality of cube}
}
\details{
Uses  a rather kludgy (but vectorized) method.  I am not 100\% sure that
the method does in fact produce magic squares for all orders and dimensions.
}
\author{Robin K. S. Hankin}
\examples{
magichypercube.4n(1,d=4)
magichypercube.4n(2,d=3)
}
\keyword{array}

\eof
\name{magicplot}
\alias{magicplot}
\title{Joins consecutive numbers of a magic square.}
\description{
A nice way to graphically represent normal magic squares.  Lines are
plotted to join successive numbers from 1 to \eqn{n^2}{n^2}.
Many magic squares have attractive such plots.
}
\usage{
magicplot(m, number = TRUE, do.circuit = FALSE, ...)
}
\arguments{
  \item{m}{Magic square to be plotted.}
  \item{number}{Boolean variable with TRUE meaning to include the
    numbers on the plot}
  \item{do.circuit}{Boolean variable with TRUE meaning to include the
    line joining \eqn{n^2}{n^2} to 1}
  \item{\dots}{Extra parameters passed to \code{plot()}}
}
\author{Robin K. S. Hankin}
\examples{
magicplot(magic.4n(2))
}
\keyword{array}


\eof
\name{min.max}
\alias{min.max}
\title{are all elements of a vector identical?}
\description{
Returns TRUE if and only if all elements of a vector are identical.
}
\usage{
min.max(x)
}
\arguments{
  \item{x}{Vector to be tested}
}
\author{Robin K. S. Hankin}
\seealso{is.magic()}
\keyword{array}


\eof
\name{notmagic.2n}
\alias{notmagic.2n}
\title{An unmagic square}
\description{
  Returns a square of order \eqn{n=2m}{n=2m} that has been claimed to
  be magic, but isn't.
}
\usage{
notmagic.2n(m)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{m}{Order of square is \eqn{n=2m}{n=2m}}
}
\references{"Magic Squares and Cubes", Andrews, (book)}
\author{Robin K. S. Hankin}
\note{This took me a whole evening to code up.  And I was quite pleased
  with the final vectorized form: it matches Andrews's (8 by 8) example
  square exactly.  What a crock.}
\examples{
  notmagic.2n(4)
  is.magic(notmagic.2n(4))
  is.semimagic(notmagic.2n(4))

}
\keyword{array}


\eof
\name{panmagic.4}
\alias{panmagic.4}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Panmagic squares of order 4}
\description{
Creates all fundamentally different panmagic squares of order 4.
}
\usage{
panmagic.4(vals = 2^(0:3))
}
\arguments{
  \item{vals}{a length four vector giving the values which are combined
    in each of the \eqn{2^4}{2^4} possible ways.  Thus
    \code{vals=2^sample(0:3)} always gives a normal square
    (0-15 in binary).}
}

\references{http://www.grogono.com/magic/index.php}
\author{Robin K. S. Hankin}
\examples{
panmagic.4()
panmagic.4(2^c(1,3,2,0))
panmagic.4(10^(0:3))
}
\keyword{array}

\eof
\name{panmagic.8}
\alias{panmagic.8}
\title{Panmagic squares of order 8}
\description{
Produces each of a wide class of order 8 panmagic squares
}
\usage{
panmagic.8(chosen = 1:6, vals = 2^(0:5))
}
\arguments{
  \item{chosen}{Which of the magic carpets are used in combination}
  \item{vals}{The values combined to produce the magic square.  Choosing
  \code{0:5} gives a normal magic square.}
}
\references{\code{http://www.grogono.com/magic/index.php}}
\author{Robin K. S. Hankin}
\note{
  Not all choices for \code{chosen} give normal magic squares.  There
  seems to be no clear pattern.  See website in references for details.
}
\seealso{\code{\link{panmagic.4}}}
\examples{
is.panmagic(panmagic.8(chosen=2:7))
is.normal(panmagic.8(chosen=2:7))
is.normal(panmagic.8(chosen=c(1,2,3,6,7,8)))

#to see the twelve basis magic carpets, use "chosen" an integer
#between 1 and 12, with vals=1:

panmagic.8(chosen=1,vals=1)-1
image(panmagic.8(chosen=12,vals=1))

}
\keyword{array}


\eof
\name{process}
\alias{process}
\title{Force index arrays into range}
\description{
Forces an (integer) array to have entries in the range 1-n, by (i)
taking the entries modulo n, then (ii) setting zero elements to n.
Useful for modifying index arrays into a form suitable for use with
magic squares.}
\usage{
process(x, n)
}
\arguments{
  \item{x}{Index array to be processed}
  \item{n}{Modulo of arithmetic to be used}
}
\author{Robin K. S. Hankin}
\examples{
# extract the broken diagonal of magic.2np1(4) that passes
# through element [1,5]:

a <- magic.2np1(4)
index <- t(c(1,5)+rbind(1:9,1:9))
a[process(index,9)]
}
\keyword{array}

\eof
